# -*- coding: utf-8 -*-

import wx
import wx.lib.calendar
import wx.grid as gridlib
import wx.lib.masked
import wx.xrc as xrc
from Hercules.exercises import *
from Hercules.workouts import *

class WorkoutsTab:
    """Workouts class that bundling all widgets"""
    
    def __init__(self, parent, resource):
        self.parent = parent
        
        # set up workouts calendar
        self.calend = wx.lib.calendar.Calendar(self.parent, wx.ID_ANY,
                                               style=wx.SUNKEN_BORDER)
        resource.AttachUnknownControl('workoutsCalendar',
                                      self.calend, self.parent)
        # set attributes of calendar
        self.calend.HideTitle()
        self.calend.HideGrid()
        self.calend.SetWeekColor('WHITE', 'BLACK')
        # month list from DateTime module
        startMonth = self.calend.GetMonth() # get the current month & year
        startYear = self.calend.GetYear()
        # set start month and year
        self.calend.SetMonth(startMonth)
        self.calend.SetYear(startYear)
        self.calend.SetCurrentDay()
        self.calend.Refresh()
        
        self.date = xrc.XRCCTRL(self.parent, 'calMonth')
        self.date.SetSelection(startMonth - 1)
        self.parent.Bind(wx.EVT_COMBOBOX, self.OnMonthCombo, self.date)
        
        self.year = xrc.XRCCTRL(self.parent, 'calYear')
        self.year.SetValue(startYear)
        self.parent.Bind(wx.EVT_SPINCTRL, self.OnYearSpin, self.year)

        # set up buttons
        # goto buttons
        self.gotoFirst = xrc.XRCCTRL(self.parent, 'gotoFirst')
        self.gotoBack = xrc.XRCCTRL(self.parent, 'gotoBack')
        self.gotoForward = xrc.XRCCTRL(self.parent, 'gotoForward')
        self.gotoLast = xrc.XRCCTRL(self.parent, 'gotoLast')
        # delete button
        self.deleteBtn = xrc.XRCCTRL(self.parent, 'deleteWorkout')
        # bind buttons events
        self.parent.Bind(wx.EVT_BUTTON, self.OnDelete, self.deleteBtn)

        # get masked controls start/finish time
        self.startTime = xrc.XRCCTRL(self.parent, 'startTime')
        self.finishTime = xrc.XRCCTRL(self.parent, 'finishTime')
        # insert grid table
        self.workoutTable = WorkoutTableGrid(self.parent,
                                             self.startTime,
                                             self.finishTime,
                                             self.calend,
                                             self.deleteBtn,
                                             resource)

        # set current date
        self.workoutDate = xrc.XRCCTRL(self.parent, 'workoutDate')
        currDate = self.calend.GetDate()
        currDate = '%02d/%02d/%d' % (currDate[0], currDate[1], currDate[2])
        self.workoutDate.SetLabel(currDate)
        
        # update workout table
        self.workoutTable.ReLoadWorkout()
        # show workouts day on a calendar
        self.workoutTable.CalendarUpdate()
        
        # bind event handlers
        self.parent.Bind(wx.lib.calendar.EVT_CALENDAR,
                         self.OnCalendarSelect,
                         self.calend)
        # self.parent.Bind(wx.EVT_TEXT,
        #                  self.OnTimeUpdate,
        #                  self.startTime)

    def OnCalendarSelect(self, event):
        """Calendar date changer"""
        currDate = '%02d/%02d/%d' % (event.day, event.month, event.year)
        self.workoutDate.SetLabel(currDate)
        self.workoutTable.ReLoadWorkout()        
        self.workoutTable.CalendarUpdate()
        self.calend.Refresh()

    def OnTimeUpdate(self, event):
        self.workoutTable.TimeUpdate()

    def OnYearSpin(self, event):
        year = self.year.GetValue()
        self.calend.SetYear(year)
        self.workoutTable.CalendarUpdate()
        self.calend.Refresh()

    def OnMonthCombo(self, event):
        name = event.GetString()
        monthVal = self.date.FindString(name)
        self.calend.SetMonth(monthVal + 1)
        self.workoutTable.CalendarUpdate()
        self.calend.Refresh()

    def OnDelete(self, event):
        self.workoutTable.DeleteWorkout()


class WorkoutTime(wx.Panel):
    """Masked control for workout start/finish time"""

    def __init__(self):
        p = wx.PrePanel()
        # the Create step is done by XRC.
        self.PostCreate(p)
        self.Bind(wx.EVT_WINDOW_CREATE, self.OnCreate)

    def OnCreate(self, event):
        """Create masked control for time control"""
        self.timeCtrl = wx.lib.masked.TimeCtrl(self, fmt24hr=True,
                                               display_seconds=False)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.timeCtrl)
        self.SetSizerAndFit(sizer)
        self.GetParent().Layout()


class WorkoutDataTable(gridlib.PyGridTableBase, Exercises, Workouts):
    """Workout data table"""

    def __init__(self, parent, startTime, finishTime, calendar,
                 delBtn):
        gridlib.PyGridTableBase.__init__(self)
        Exercises.__init__(self)
        Workouts.__init__(self)

        # set access to parent controls
        self.startTime = startTime
        self.finishTime = finishTime
        self.cal = calendar
        # buttons
        self.delBtn = delBtn

        self.colLabels = [_("Title"), _("Type"), _("Set"),
                          _("Reps"), _("Weight")]

        # init title choices
        self.exerciseList = []
        self.titleID = [0]
        exercises = self.SelectExercises()
        titleChoices = [""]
        if list(exercises) != []:
            for p in exercises:
                titleChoices.append(p.exerTitle)
                self.titleID.append(p.id)
            self.exerciseList = titleChoices
        tmpChoices = ':' + titleChoices[0]
        for n in xrange(1, len(titleChoices)):
            tmpChoices = tmpChoices + ',' + titleChoices[n]
        titleChoices = tmpChoices

        self.setTypes = [_("Warmup"), _("Working")]

        setChoices = ':' + self.setTypes[0] + ',' + self.setTypes[1]
        
        self.dataTypes = [gridlib.GRID_VALUE_CHOICE + titleChoices,
                          gridlib.GRID_VALUE_CHOICE + setChoices,
                          gridlib.GRID_VALUE_NUMBER + ':1,8',
                          gridlib.GRID_VALUE_NUMBER + ':1,30',
                          gridlib.GRID_VALUE_FLOAT + ':3,2',
                          ]

        self.data = [["", "", "", "", ""]]
        self.dataID = []
        # check if we load the table
        self.isLoadTable = False

    #--------------------------------------------------
    # required methods for the wxPyGridTableBase interface

    def GetNumberRows(self):
        return len(self.data) + 1

    def GetNumberCols(self):
        return len(self.data[0])

    def IsEmptyCell(self, row, col):
        try:
            return not self.data[row][col]
        except IndexError:
            return True

    # Get/Set values in the table.  The Python version of these
    # methods can handle any data-type, (as long as the Editor and
    # Renderer understands the type too,) not just strings as in the
    # C++ version.
    def GetValue(self, row, col):
        try:
            return self.data[row][col]
        except IndexError:
            return ''

    def SetValue(self, row, col, value):
        try:
            insertUpdate = False
            if self.isLoadTable == False:
                if self.GetValue(row, 0) == '' and col == 0:
                    if self.GetValue(row - 1, 0) != '' \
                           or row == 0:
                        self.data[row][col] = value
                        self.data[row][1] = self.setTypes[0]
                        self.data[row][2] = 1
                        self.data[row][3] = 1
                        self.data[row][4] = 0.0
                        insertUpdate = True
                elif self.GetValue(row, 0) != '':
                    self.data[row][col] = value
                    insertUpdate = True
                if insertUpdate == True:
                    self.InsertRow(row)
            else:
                self.data[row][col] = value
        except IndexError:
            # add a new row
            self.data.append([''] * self.GetNumberCols())
            self.dataID.append([] * self.GetNumberCols())
            self.SetValue(row, col, value)
            
            # tell the grid we've added a row
            msg = gridlib.GridTableMessage(self,            
                gridlib.GRIDTABLE_NOTIFY_ROWS_APPENDED, 1)
            
            self.GetView().ProcessTableMessage(msg)

    def InsertRow(self, row):
        try:
            # update existed set
            if self.dataID[row] == []:
                raise SQLObjectNotFound
            oldSet = self.GetSet(self.dataID[row])
            self.UpdateSet(oldSet, self.titleID[self.exerciseList.index(self.data[row][0])],
                           self.setTypes.index(self.data[row][1]),
                           self.data[row][2], self.data[row][3],
                           self.data[row][4])
        except SQLObjectNotFound:
            if self.GetValue(row, 0) != '':
                # insert new set
                workoutDate = self.cal.GetDate()
                workoutDate = '%d-%02d-%02d' % (workoutDate[2], \
                                                workoutDate[1], workoutDate[0])
                workout = self.GetWorkout(workoutDate)
                if workout == []:
                    workout = self.InsertWorkout(workoutDate,
                                                 0, 0)
                    #self.startTime.GetValue(),
                    #self.finishTime.GetValue())
                    # enable delete button
                    self.delBtn.Enable()
                newSet = self.InsertSet(self.titleID[self.exerciseList.index(self.data[row][0])],
                                self.setTypes.index(self.data[row][1]),
                                self.data[row][2], self.data[row][3],
                                self.data[row][4], workout)
                # store set id
                self.dataID[row] = newSet.id
                
    def DeleteRow(self, row):
        # delete exercise from database
        oldSet = self.GetSet(self.dataID[row])
        self.DeleteSet(oldSet)
        # delete exercise from table
        del self.data[row]
        # delete id of exercise
        del self.dataID[row]
        # update table
        msg = gridlib.GridTableMessage(self,
                    gridlib.GRIDTABLE_NOTIFY_ROWS_DELETED,
                    row, 1)
        self.GetView().ProcessTableMessage(msg)
        if len(self.data) == 0:
            # if this is last set delete workout entirely
            workoutDate = self.cal.GetDate()
            workoutDate = '%d-%02d-%02d' % (workoutDate[2], \
                                            workoutDate[1], workoutDate[0])
            workout = self.GetWorkout(workoutDate)
            self.DeleteWorkout(workout)
            # insert empty lines
            msg = gridlib.GridTableMessage(self,            
                gridlib.GRIDTABLE_NOTIFY_ROWS_APPENDED, 1)
            self.GetView().ProcessTableMessage(msg)
            self.data = [["", "", "", "", ""]]
            self.dataID = []
            # disable delete button
            self.delBtn.Disable()

    def CleanTable(self):
        """Clean workout table"""
        # clean table if there is some data
        if len(self.data) >= 1:
            msg = gridlib.GridTableMessage(self,
                        gridlib.GRIDTABLE_NOTIFY_ROWS_DELETED,
                        0, self.GetNumberRows())
            self.GetView().ProcessTableMessage(msg)
            # insert empty lines
            msg = gridlib.GridTableMessage(self,            
                gridlib.GRIDTABLE_NOTIFY_ROWS_APPENDED, 2)
            self.GetView().ProcessTableMessage(msg)
            # set up empty variables
            self.data = [["", "", "", "", ""]]
            self.dataID = []
            self.isLoadTable = False
            # disable delete button
            self.delBtn.Disable()

    def LoadTable(self):
        """Load table"""
        selDate = self.cal.GetDate()
        selDate = '%d-%02d-%02d' % (selDate[2], selDate[1], selDate[0])
        workout, sets = self.SelectWorkout(selDate)
        if list(sets) != []:
            self.data = [["", "", "", "", ""]]
            self.dataID = []

            self.isLoadTable = True

            tmpData = []
            for p in sets:
                tmpData.append([p.setTitle, p.setType,
                                p.setNum, p.setReps, p.setWeight])
                self.dataID.append(p.id)
            for n in xrange(0, len(tmpData)):
                self.SetValue(n, 0, self.exerciseList[tmpData[n][0]])
                self.SetValue(n, 1, self.setTypes[tmpData[n][1]])
                self.SetValue(n, 2, tmpData[n][2])
                self.SetValue(n, 3, tmpData[n][3])
                self.SetValue(n, 4, tmpData[n][4])

            # set stored time
            # p = workout[0]
            # startTime = p.workoutStartTime
            # finishTime = p.workoutFinishTime

            # enable delete button 
            self.delBtn.Enable()

            self.isLoadTable = False

    def DeleteTable(self):
        """Delete workout table"""
        workoutDate = self.cal.GetDate()
        workoutDate = '%d-%02d-%02d' % (workoutDate[2], \
                                        workoutDate[1], workoutDate[0])
        workout = self.GetWorkout(workoutDate)
        if workout != []:
            self.DeleteWorkoutWithSets(workout)
            self.CleanTable()
            self.data = [["", "", "", "", ""]]
            self.dataID = []
            # disable delete button
            self.delBtn.Disable()

    def TimeUpdate(self):
        """Save workout time"""
        pass
        # workoutDate = self.cal.GetDate().FormatISODate()
        # workout = self.GetWorkout(workoutDate)
        # if workout != []:
            # self.UpdateWorkout(workout,
            #                    self.startTime.GetValue(),
            #                    self.finishTime.GetValue())
            # self.startTime.GetValue()
            # self.finishTime.GetValue()

    def CalendarUpdate(self):
        dateList = []
        currYear = self.cal.GetYear()
        currMonth = self.cal.GetMonth()
        for d in xrange(1, 31):
            date = '%d-%02d-%02d' % (currYear, currMonth, d)
            workout = self.GetWorkout(date)
            if workout != []:
                dateList.append(d)
        self.cal.SetSelDay(dateList)

    #--------------------------------------------------
    # Some optional methods

    # Called when the grid needs to display labels
    def GetColLabelValue(self, col):
        return self.colLabels[col]


    # Called to determine the kind of editor/renderer to use by
    # default, doesn't necessarily have to be the same type used
    # natively by the editor/renderer if they know how to convert.
    def GetTypeName(self, row, col):
        return self.dataTypes[col]

    # Called to determine how the data can be fetched and stored by
    # the editor and renderer.  This allows you to enforce some
    # type-safety in the grid.
    def CanGetValueAs(self, row, col, typeName):
        colType = self.dataTypes[col].split(':')[0]
        if typeName == colType:
            return True
        else:
            return False

    def CanSetValueAs(self, row, col, typeName):
        return self.CanGetValueAs(row, col, typeName)


class WorkoutTableGrid(gridlib.Grid):
    """Workout table grid"""
    
    def __init__(self, parent, startTime, finishTime, calendar,
                 delBtn, resource):
        gridlib.Grid.__init__(self, parent, style=wx.SUNKEN_BORDER)
        resource.AttachUnknownControl('workoutTable', self, parent)

        self.table = WorkoutDataTable(parent, startTime, finishTime,
                                      calendar, delBtn)

        # The second parameter means that the grid is to take
        # ownership of the table and will destroy it when done.
        # Otherwise you would need to keep a reference to it and call
        # it's Destroy method later.
        self.SetTable(self.table, True)

        self.SetRowLabelSize(0)
        self.SetMargins(0,0)
        self.AutoSizeColumns(False)
        self.SetColSize(0,250)
        self.SetColSize(1,100)
        self.SetColSize(2,50)
        self.SetColSize(3,50)
        self.SetColSize(4,75)
        self.DisableDragRowSize()
        
        gridlib.EVT_GRID_CELL_LEFT_DCLICK(self, self.OnLeftDClick)
        # gridlib.EVT_GRID_EDITOR_SHOWN(self, self.OnChoiceUpdate)

        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
            
    # I do this because I don't like the default behaviour of not
    # starting the cell editor on double clicks, but only a second
    # click.
    def OnLeftDClick(self, evt):
        if self.CanEnableCellControl():
            self.EnableCellEditControl()

    def OnKeyDown(self, evt):
        if evt.KeyCode() != wx.WXK_DELETE and \
               evt.KeyCode() != wx.WXK_RETURN:
            evt.Skip()
            return

        if evt.ControlDown():   # the edit control needs this key
            evt.Skip()
            return

        # swith to editing mode
        if evt.KeyCode() == wx.WXK_RETURN:
            if self.IsCellEditControlEnabled() != True:
                self.EnableCellEditControl()
            else:
                self.DisableCellEditControl()

        # delete selected row
        if evt.KeyCode() == wx.WXK_DELETE:
            if self.IsCellEditControlEnabled() != True:
                table = self.GetTable()
                if table.GetValue(self.GetGridCursorRow(), 0) != '':
                    table.DeleteRow(self.GetGridCursorRow())

    def OnChoiceUpdate(self, event):
        """Update exercises list in the combo box"""
        print event.GetControl()

    def ReLoadWorkout(self):
        """Load or Reload workout table"""
        self.table.CleanTable()
        self.table.LoadTable()

    def DeleteWorkout(self):
        self.table.DeleteTable()

    def TimeUpdate(self):
        self.table.TimeUpdate()

    def CalendarUpdate(self):
        self.table.CalendarUpdate()
